==> OpenCL-0.8.3.tgz: OpenCL-0.8.3/src/eme1.cpp <==
/*************************************************
* EME1 Source File                               *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/eme1.h>
#include <opencl/mgf1.h>
#include <opencl/rng.h>

namespace OpenCL {

/*************************************************
* EME1 Pad Operation                             *
*************************************************/
SecureVector<byte> EME1::pad(const byte in[], u32bit inlen,
                             u32bit olen) const
   {
   olen /= 8;

   if(inlen > olen - 2*HASH_LENGTH - 1)
      throw Exception("EME1: Input is too large");

   SecureVector<byte> out(olen);

   do
      {
      out.clear();
      Global_RNG::randomize(out, HASH_LENGTH);
      out.copy(HASH_LENGTH, Phash, Phash.size());
      out[out.size() - inlen - 1] = 0x01;
      out.copy(out.size() - inlen, in, inlen);

      mgf->mask(out, HASH_LENGTH, out + HASH_LENGTH, out.size() - HASH_LENGTH);
      mgf->mask(out + HASH_LENGTH, out.size() - HASH_LENGTH, out, HASH_LENGTH);
      } while(out[0] == 0);
   return out;
   }

/*************************************************
* EME1 Unpad Operation                           *
*************************************************/
SecureVector<byte> EME1::unpad(const byte in[], u32bit inlen,
                               u32bit key_len) const
   {
   if(inlen != key_len / 8)
      throw Decoding_Error("EME1::unpad");

   SecureVector<byte> retval;
   if(inlen <= 2*HASH_LENGTH + 1)
      return retval;

   SecureVector<byte> tmp(in, inlen);

   mgf->mask(tmp + HASH_LENGTH, tmp.size() - HASH_LENGTH, tmp, HASH_LENGTH);
   mgf->mask(tmp, HASH_LENGTH, tmp + HASH_LENGTH, tmp.size() - HASH_LENGTH);

   for(u32bit j = 0; j != Phash.size(); j++)
      if(tmp[j+HASH_LENGTH] != Phash[j])
         throw Decoding_Error("EME1::unpad");

   for(u32bit j = HASH_LENGTH + Phash.size(); j != tmp.size(); j++)
      {
      if(tmp[j] && tmp[j] != 0x01)
         throw Decoding_Error("EME1::unpad");
      if(tmp[j] && tmp[j] == 0x01)
         {
         retval.set_to(tmp + j + 1, tmp.size() - j - 1);
         return retval;
         }
      }
   throw Decoding_Error("EME1::unpad");
   }

/*************************************************
* Return the max input size for a given key size *
*************************************************/
u32bit EME1::maximum_input_size(u32bit keybits) const
   {
   if(keybits / 8 > 2*HASH_LENGTH + 1)
      return ((keybits / 8) - 2*HASH_LENGTH - 1);
   else
      return 0;
   }

/*************************************************
* EME1 Constructor                               *
*************************************************/
EME1::EME1(HashFunction* h, MaskGenerationFunction* m, const std::string& P) :
   HASH_LENGTH(h->OUTPUT_LENGTH), mgf(m ? m : new MGF1(h->clone()))
   {
   h->clear();
   Phash = h->process(P);
   delete h;
   }

}

==> OpenCL-0.8.4.tgz: OpenCL-0.8.4/src/eme1.cpp <==
/*************************************************
* EME1 Source File                               *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/eme1.h>
#include <opencl/mgf1.h>
#include <opencl/rng.h>

namespace OpenCL {

/*************************************************
* EME1 Pad Operation                             *
*************************************************/
SecureVector<byte> EME1::pad(const byte in[], u32bit inlen,
                             u32bit olen) const
   {
   olen /= 8;

   if(inlen > olen - 2*HASH_LENGTH - 1)
      throw Exception("EME1: Input is too large");

   SecureVector<byte> out(olen);

   do
      {
      out.clear();
      Global_RNG::randomize(out, HASH_LENGTH);
      out.copy(HASH_LENGTH, Phash, Phash.size());
      out[out.size() - inlen - 1] = 0x01;
      out.copy(out.size() - inlen, in, inlen);

      mgf->mask(out, HASH_LENGTH, out + HASH_LENGTH, out.size() - HASH_LENGTH);
      mgf->mask(out + HASH_LENGTH, out.size() - HASH_LENGTH, out, HASH_LENGTH);
      } while(out[0] == 0);
   return out;
   }

/*************************************************
* EME1 Unpad Operation                           *
*************************************************/
SecureVector<byte> EME1::unpad(const byte in[], u32bit inlen,
                               u32bit key_len) const
   {
   if(inlen != key_len / 8)
      throw Decoding_Error("EME1::unpad");

   SecureVector<byte> retval;
   if(inlen <= 2*HASH_LENGTH + 1)
      return retval;

   SecureVector<byte> tmp(in, inlen);

   mgf->mask(tmp + HASH_LENGTH, tmp.size() - HASH_LENGTH, tmp, HASH_LENGTH);
   mgf->mask(tmp, HASH_LENGTH, tmp + HASH_LENGTH, tmp.size() - HASH_LENGTH);

   for(u32bit j = 0; j != Phash.size(); j++)
      if(tmp[j+HASH_LENGTH] != Phash[j])
         throw Decoding_Error("EME1::unpad");

   for(u32bit j = HASH_LENGTH + Phash.size(); j != tmp.size(); j++)
      {
      if(tmp[j] && tmp[j] != 0x01)
         throw Decoding_Error("EME1::unpad");
      if(tmp[j] && tmp[j] == 0x01)
         {
         retval.set(tmp + j + 1, tmp.size() - j - 1);
         return retval;
         }
      }
   throw Decoding_Error("EME1::unpad");
   }

/*************************************************
* Return the max input size for a given key size *
*************************************************/
u32bit EME1::maximum_input_size(u32bit keybits) const
   {
   if(keybits / 8 > 2*HASH_LENGTH + 1)
      return ((keybits / 8) - 2*HASH_LENGTH - 1);
   else
      return 0;
   }

/*************************************************
* EME1 Constructor                               *
*************************************************/
EME1::EME1(const std::string& hash_name, const std::string& mgf_name,
           const std::string& P) :
   HASH_LENGTH(output_length_of(hash_name))
   {
   if(mgf_name != "MGF1")
      throw Algorithm_Not_Found(mgf_name);

   HashFunction* hash = get_hash(hash_name);
   mgf = new MGF1(hash->name());
   Phash = hash->process(P);
   delete hash;
   }

/*************************************************
* EME1 Constructor                               *
*************************************************/
EME1::EME1(HashFunction* h, MGF* m, const std::string& P) :
   HASH_LENGTH(h->OUTPUT_LENGTH), mgf(m ? m : new MGF1(h->name()))
   {
   h->clear();
   Phash = h->process(P);
   delete h;
   }

}
